/* 
 * Flow.h
 * Copyright (c) 2004 Vlad GALU <dudu@rdsnet.ro>
 *		      Andrei GAVRILOAIE <gavriloaie_andrei@yahoo.com>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef FLOW_H
#define FLOW_H

class CFlowNode {
public:
	flow_t m_flow;
	u_int key[4];

	CFlowNode *m_pLeft;
	CFlowNode *m_pRight;
	CFlowNode *m_pParent;
public:
	CFlowNode(flow_t &flow);
	CFlowNode(
		in_addr &saddr,
		in_addr &daddr,
		u_short &sport,
		u_short &dport,
		u_char &tcp_flags,
		u_char &proto,
		u_char &tos,
		u_int &bytes,
		time_t &now);
	~CFlowNode(void);

	bool operator < (CFlowNode &node);
	bool operator > (CFlowNode &node);
	bool operator == (CFlowNode &node);
	bool operator <= (CFlowNode &node);
	bool operator >= (CFlowNode &node);
#ifdef _DEBUG
	void show(int &nrNodes);
#endif //_DEBUG
};


class CFlowTree {
private:
	CFlowNode *m_pRoot;
	u_int m_numberOfNodes;
private:
	bool insertNode(CFlowNode *pNode);
	void deleteNode(CFlowNode *pNode);
public:
	CFlowTree();
	~CFlowTree();
	bool insertNode(flow_t &flow);
	bool insertNode(
		in_addr &saddr,
		in_addr &daddr,
		u_short &sport,
		u_short &dport,
		u_char &tcp_flags,
		u_char &proto,
		u_char &tos,
		u_int &bytes,
		time_t &now);
	void cleanUp(CLEANFUNC cleanfunc);
	u_int getNumberOfNodes();
#ifdef _DEBUG
	int show();
#endif //_DEBUG
};

#endif //FLOW_H


/* :set ts=4 */

